@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -53,6 +53,7 @@ import static com.sun.tools.javac.code.Flags.SYNTHETIC;
 import static com.sun.tools.javac.code.Kinds.Kind.MDL;
 import static com.sun.tools.javac.code.Kinds.Kind.MTH;
 import static com.sun.tools.javac.code.Kinds.Kind.PCK;
+import static com.sun.tools.javac.code.Kinds.Kind.TYP;
 import static com.sun.tools.javac.code.Kinds.Kind.VAR;
 import static com.sun.tools.javac.code.Scope.LookupKind.NON_RECURSIVE;
 import static com.sun.tools.javac.code.TypeTag.ARRAY;
@@ -369,14 +370,19 @@ public class Annotate {
                 }
             }
 
-            // Note: @Deprecated has no effect on local variables and parameters
             if (!c.type.isErroneous()
                     && types.isSameType(c.type, syms.previewFeatureType)) {
                 toAnnotate.flags_field |= Flags.PREVIEW_API;
-                if (isAttributeTrue(c.member(names.essentialAPI))) {
-                    toAnnotate.flags_field |= Flags.PREVIEW_ESSENTIAL_API;
+                if (isAttributeTrue(c.member(names.reflective))) {
+                    toAnnotate.flags_field |= Flags.PREVIEW_REFLECTIVE;
                 }
             }
+
+            if (!c.type.isErroneous()
+                    && toAnnotate.kind == TYP
+                    && types.isSameType(c.type, syms.valueBasedType)) {
+                toAnnotate.flags_field |= Flags.VALUE_BASED;
+            }
         }
 
         List<T> buf = List.nil();
@@ -404,13 +410,9 @@ public class Annotate {
     }
     //where:
         private boolean isAttributeTrue(Attribute attr) {
-            if (attr instanceof Attribute.Constant) {
-                Attribute.Constant v = (Attribute.Constant) attr;
-                if (v.type == syms.booleanType && ((Integer) v.value) != 0) {
-                    return true;
-                }
-            }
-            return false;
+            return (attr instanceof Attribute.Constant constant)
+                    && constant.type == syms.booleanType
+                    && ((Integer) constant.value) != 0;
         }
 
     /**
@@ -424,7 +426,7 @@ public class Annotate {
     public Attribute.Compound attributeAnnotation(JCAnnotation tree, Type expectedAnnotationType,
                                                   Env<AttrContext> env)
     {
-        // The attribute might have been entered if it is Target or Repetable
+        // The attribute might have been entered if it is Target or Repeatable
         // Because TreeCopier does not copy type, redo this if type is null
         if (tree.attribute != null && tree.type != null)
             return tree.attribute;
@@ -445,9 +447,9 @@ public class Annotate {
     public Attribute.TypeCompound attributeTypeAnnotation(JCAnnotation a, Type expectedAnnotationType,
                                                           Env<AttrContext> env)
     {
-        // The attribute might have been entered if it is Target or Repetable
+        // The attribute might have been entered if it is Target or Repeatable
         // Because TreeCopier does not copy type, redo this if type is null
-        if (a.attribute == null || a.type == null || !(a.attribute instanceof Attribute.TypeCompound)) {
+        if (a.attribute == null || a.type == null || !(a.attribute instanceof Attribute.TypeCompound typeCompound)) {
             // Create a new TypeCompound
             List<Pair<MethodSymbol,Attribute>> elems =
                     attributeAnnotationValues(a, expectedAnnotationType, env);
@@ -458,7 +460,7 @@ public class Annotate {
             return tc;
         } else {
             // Use an existing TypeCompound
-            return (Attribute.TypeCompound)a.attribute;
+            return typeCompound;
         }
     }
 
@@ -540,7 +542,7 @@ public class Annotate {
     private Attribute attributeAnnotationValue(Type expectedElementType, JCExpression tree,
             Env<AttrContext> env)
     {
-        //first, try completing the symbol for the annotation value - if acompletion
+        //first, try completing the symbol for the annotation value - if a completion
         //error is thrown, we should recover gracefully, and display an
         //ordinary resolution diagnostic.
         try {
@@ -911,12 +913,12 @@ public class Annotate {
             log.error(pos, Errors.InvalidRepeatableAnnotation(annoDecl));
             return null;
         }
-        if (!(p.snd instanceof Attribute.Class)) { // check that the value of "value" is an Attribute.Class
+        if (!(p.snd instanceof Attribute.Class attributeClass)) { // check that the value of "value" is an Attribute.Class
             log.error(pos, Errors.InvalidRepeatableAnnotation(annoDecl));
             return null;
         }
 
-        return ((Attribute.Class)p.snd).getValue();
+        return attributeClass.getValue();
     }
 
     /* Validate that the suggested targetContainerType Type is a valid
@@ -1127,7 +1129,7 @@ public class Annotate {
 
         @Override
         public void visitBindingPattern(JCTree.JCBindingPattern tree) {
-            //type binding pattern's type will be annotated separatelly, avoid
+            //type binding pattern's type will be annotated separately, avoid
             //adding its annotations into the owning method here (would clash
             //with repeatable annotations).
         }
@@ -1164,7 +1166,7 @@ public class Annotate {
     };
 
     /* Last stage completer to enter just enough annotations to have a prototype annotation type.
-     * This currently means entering @Target and @Repetable.
+     * This currently means entering @Target and @Repeatable.
      */
     public AnnotationTypeCompleter annotationTypeSourceCompleter() {
         return theSourceCompleter;
